/* 
 * Field Widget for GTK
 * Copyright (C) 2007 Milko Krachounov <exabyte@3mhz.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* This is a subclass of GtkLabel that tries to go around this bug:
            http://bugzilla.gnome.org/show_bug.cgi?id=101968

   The idea came from VMWare's libview WrapLabel, from http://view.sourceforge.net
   
   When using the widget in tables, use GTK_FILL | GTK_EXPAND as GtkAttachOptions

   We provide new size_request and size_allocate handlers.
   We request the current layout height as height and hope we get the biggest possible
   width, then on allocation we try to change the pango layout to match the width,
   and we request a widget resize, to accomodate the new height.
 */

#include "fieldlabel.h"

static void     moko_field_label_size_request   (GtkWidget		*widget,
					    GtkRequisition	*requisition);
static void     moko_field_label_size_allocate  (GtkWidget		*widget,
					    GtkAllocation	*allocation);
static void	moko_field_label_set_wrap_width (MokoFieldLabel		*moko_field_label,
					    size_t		width);



G_DEFINE_TYPE (MokoFieldLabel, moko_field_label, GTK_TYPE_LABEL)


static void
moko_field_label_class_init (MokoFieldLabelClass *class)
{
	GtkWidgetClass *widget_class;
	widget_class = (GtkWidgetClass*) class;
	widget_class->size_request = moko_field_label_size_request;
	widget_class->size_allocate = moko_field_label_size_allocate;

}


static void
moko_field_label_init (MokoFieldLabel *moko_field_label)
{
	g_return_if_fail (MOKO_IS_FIELD_LABEL (moko_field_label));
	
	moko_field_label->wrap_width = 0;

	
}

GtkWidget *
moko_field_label_new (const gchar* text)
{
	MokoFieldLabel *moko_field_label;

	moko_field_label = g_object_new (MOKO_TYPE_FIELD_LABEL, NULL);
	
	pango_layout_set_wrap (gtk_label_get_layout(GTK_LABEL(moko_field_label)), PANGO_WRAP_WORD_CHAR);
	gtk_misc_set_alignment (GTK_MISC(moko_field_label), 0.0f, 0.0f);
	if (text && *text)
		moko_field_label_set_text (moko_field_label, text);


	return GTK_WIDGET (moko_field_label);
}

void
moko_field_label_set_text (MokoFieldLabel 	*moko_field_label, 
		      const gchar	*text)
{
	g_return_if_fail (MOKO_IS_FIELD_LABEL (moko_field_label));
	gtk_label_set_text (GTK_LABEL(moko_field_label), text);
	moko_field_label_set_wrap_width (moko_field_label, moko_field_label->wrap_width);
}

static void
moko_field_label_set_wrap_width (MokoFieldLabel 	*moko_field_label,
			    size_t	width)
{
	if (width == 0)
		return;
	pango_layout_set_width(gtk_label_get_layout(GTK_LABEL(moko_field_label)), width * PANGO_SCALE);
	if (width != moko_field_label->wrap_width) {
		moko_field_label->wrap_width = width;
		gtk_widget_queue_resize(GTK_WIDGET(moko_field_label));
	}
	
}

static void
moko_field_label_size_request (GtkWidget      *widget,
			  GtkRequisition *requisition)
{
	int width = 0, height = 0;
	pango_layout_get_pixel_size(gtk_label_get_layout(GTK_LABEL(widget)), &width, &height);
	requisition->width = 0;
	requisition->height = height;
}

static void
moko_field_label_size_allocate (GtkWidget     *widget,
			   GtkAllocation *allocation)
{
	GtkWidgetClass *widget_class;
	widget_class = gtk_type_class(GTK_TYPE_WIDGET);
	widget_class->size_allocate(widget, allocation);
	moko_field_label_set_wrap_width(MOKO_FIELD_LABEL(widget), allocation->width);

}

#if 0
 void
moko_field_label_set_markup     (MokoFieldLabel		*moko_field_label,
			    const gchar		*str)
{
	gtk_label_set_markup(GTK_LABEL(moko_field_label), str);
	moko_field_label_set_wrap_width(moko_field_label, moko_field_label->wrap_width);
}

#endif
