/*
 * Mail for OpenMoko
 * Copyright (C) 2007 Milko Krachounov <exabyte@3mhz.net>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <strings.h>
#include <string.h>
#include <moko-finger-scroll.h>
#include <moko-stock.h>

#include "mokofieldlabel.h"
#include "mail-defines.h"
#include "mail-common.h"
#include "mail-write.h"



typedef struct {
	AppData *app_data;
	display_info message;
	GtkWidget *page;
	union {
		struct {
			GtkWidget *tofrom;
		};
		struct {
			GtkWidget *to;
			GtkWidget *subject;
			GtkWidget *cc;
			GtkWidget *bcc;
			GtkWidget *attachment_box;
		};
			
	} widgets;
} MessagesWritePageData;

static void
cb_message_close (GtkToolButton *button, MessagesWritePageData *data)
{
	gint page_num;
	page_num = gtk_notebook_page_num(data->app_data->notebook, data->page);
	gtk_notebook_remove_page(data->app_data->notebook, page_num);
	display_info_unset(&(data->message));
	g_free(data);
}

static void
cb_send_sms (GtkToolButton *button, MessagesWritePageData *data)
{

}

static void
cb_send_email (GtkToolButton *button, MessagesWritePageData *data)
{

}

GtkWidget*
messages_create_write_sms_page (AppData* app_data, gint action, display_info *message)
{
	GtkWidget *page, *toolbar, *viewport, *scroll, *table, *text;
	GList *row_list = NULL;

	MessagesWritePageData *data = g_malloc0(sizeof(MessagesWritePageData));

	page = gtk_vbox_new (FALSE, 0);

	toolbar = gtk_toolbar_new ();
	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (GTK_STOCK_CLOSE, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_message_close), data);
	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (MOKO_STOCK_MAIL_SEND, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_send_sms), data);
	
	gtk_box_pack_start (GTK_BOX (page), toolbar, FALSE, TRUE, 0);


	/* This is our input */
	row_list = header_table_append_entry(row_list, _("To:"), &(data->widgets.tofrom), data->message.sms.tofrom);
	table = header_table_create(row_list, 0.5f, 3);
	gtk_box_pack_start (GTK_BOX (page), table, FALSE, TRUE, 0);

	viewport = gtk_viewport_new(NULL, NULL);
	gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);

	//message = gtk_vbox_new (FALSE, 0);
	if (!data->message.sms.message)
		data->message.sms.message = gtk_text_buffer_new(NULL);
	text = gtk_text_view_new_with_buffer(data->message.sms.message);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD_CHAR);
	gtk_container_set_border_width(GTK_CONTAINER(text), 1);

	//gtk_box_pack_start (GTK_BOX (message), text, TRUE, TRUE, 0);
	
	//viewport = gtk_viewport_new(NULL, NULL);
	

	scroll = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll), GTK_SHADOW_ETCHED_IN);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	//gtk_container_add (GTK_CONTAINER (viewport), text);	
	gtk_container_add (GTK_CONTAINER (scroll), text);
	

	gtk_box_pack_end (GTK_BOX (page), scroll, TRUE, TRUE, 0);

	data->app_data = app_data;
	data->page = page;
	
	return page;

	

}

/* Mail function */

/* The following functions are used to quote a message when forwarding or replying,
   they are currently usable only in plain text mails?
*/


/* This must be freed */
/* This function creates a short header when forwarding a message */
static gchararray
forward_mail_header(display_info *message)
{
	gsize i, j, len;
	gchararray result;
	const gchar* label_original = N_("Original letter");
	const gchar* bar = "--------";
	const gchar* date = date_human_readable(message->mail.ts);
	struct {
		const gchar* label;
		const gchar* text;
	} fields[] = {
		{ N_("Subject:"), message->mail.subject },
		{ N_("Date:"), date },
		{ N_("From:"), message->mail.from },
		{ N_("Cc:"), message->mail.cc },
		{ NULL, NULL }
	};
	len = strlen(bar)*2 + strlen(_(label_original)) + 5; /* 5 = 2 spaces + 3 newlines */
	for (i = 0; fields[i].label; i++)
		if (fields[i].text)
			len += strlen(_(fields[i].label)) + strlen(fields[i].text) + 2;
	len = len + 1; /* The NULL character at the end */
	/* Be careful here! */
	result = g_malloc(sizeof(gchar)*len);
	j = snprintf(result, len, "\n\n%s %s %s\n", bar, _(label_original), bar);
	for (i = 0; fields[i].label; i++)
		if (fields[i].text)
			j += snprintf(result + j, len - j, "%s %s\n", fields[i].label, fields[i].text);
	/* Just to be sure */
	if(result[len] != '\0')
		result[len] = '\0';

	return result;
		
}

static inline gchararray
reply_mail_header (display_info *message)
{
	return g_strdup_printf(_("\n\n%s wrote: \n"), message->mail.from);
}

static void
text_counter(const gchar * text, gsize *len, gsize *lines)
{
	gsize i, count = 0;
	for (i = 0; text[i]; i++)
		if (text[i] == '\n') count++;
	*len = i;
	*lines = count;

}

/* This must be freed */
static gchararray 
quote_and_free_text(gchararray text, const gchar* header)
{
	gsize len, lines, i, j;
	gchararray result;
	text_counter(text, &len, &lines);
	j = strlen(header);
	len = len+lines*2+j+3; /* Two characters plus null char! */
	/* Be careful here ! */
	result = g_malloc(sizeof(gchar)*len);
	strncpy(result, header, j);
	if (j < len-1)
		result[j++] = '>';
	if (j < len-1)
		result[j++] = ' ';
	for (i = 0; text[i] && j < len-1; i++, j++)
	{
		result[j] = text[i];
		if (text[i] == '\n') {
			if (text[i+1])
				result[++j] = '>';
				result[++j] = ' ';
		}
	}
	result[j] = '\0';

	g_free(text);
	return result;
}

static void
forward_inline (display_info *dst, display_info *src) {
	gchararray text, header, quoted_text;

	GtkTextIter start = {0, }, end = {0, };
		

	/* Headers */
	/* NOTE: This is not localized, is that OK ? */
	dst->mail.subject = g_strdup_printf("Fwd: %s", src->mail.subject);


	dst->mail.attachments = attachment_list_copy(src->mail.attachments);
	

	header = forward_mail_header(src);

	
	gtk_text_buffer_get_start_iter (src->mail.message, &start);
	gtk_text_buffer_get_end_iter (src->mail.message, &end);

	text = gtk_text_buffer_get_text(src->mail.message, &start, &end, FALSE);
	
	quoted_text = quote_and_free_text(text, header);

	g_free(header);

	gtk_text_buffer_set_text(dst->mail.message, quoted_text, -1);
	g_free(quoted_text);
	
}

static void
reply (display_info *dst, display_info *src, gboolean all) {
	gchararray text, header, quoted_text;

	GtkTextIter start = {0, }, end = {0, };
		
	/* Headers */
	dst->mail.subject = g_strdup_printf("Re: %s", src->mail.subject);
	dst->mail.to = g_strdup(src->mail.from);
	if (all) {
		// TODO: To field forgotten here...
		dst->mail.cc = g_strdup(src->mail.cc);
	}

	header = reply_mail_header(src);

	
	gtk_text_buffer_get_start_iter (src->mail.message, &start);
	gtk_text_buffer_get_end_iter (src->mail.message, &end);

	text = gtk_text_buffer_get_text(src->mail.message, &start, &end, FALSE);
	
	quoted_text = quote_and_free_text(text, header);
	g_free(header);

	gtk_text_buffer_set_text(dst->mail.message, quoted_text, -1);
	g_free(quoted_text);
}


/* ... */

static void redraw_attachments(MessagesWritePageData *data);
static void
cb_remove_attachment(GtkButton* button, MessagesWritePageData *data)
{
	mail_attachment *attachment;
	
	g_object_get(G_OBJECT(button), "user-data", &attachment, NULL);
	
	data->message.mail.attachments = g_list_remove(data->message.mail.attachments, attachment);
	redraw_attachments(data);
}

static void
display_attachment (mail_attachment *attachment, MessagesWritePageData *data)
{
	char buf[32];
	const gchar *unit;
	GtkWidget *hbox;
	GtkWidget *name, *size;
	GtkWidget *button;
	PangoAttrList *bold;
	PangoAttribute *attr;

	/* Bold attribute */
	bold = pango_attr_list_new();	
	attr = pango_attr_weight_new(PANGO_WEIGHT_BOLD);
	attr->start_index = 0;
	attr->end_index = G_MAXUINT;
	pango_attr_list_insert(bold, attr);

	hbox = gtk_hbox_new(FALSE, 3);
	
	name = moko_field_label_new(attachment->name);
	gtk_label_set_ellipsize(GTK_LABEL(name), PANGO_ELLIPSIZE_END);
	gtk_label_set_attributes(GTK_LABEL(name), bold);

	button = gtk_button_new();
	gtk_button_set_label(GTK_BUTTON(button), "X");
	g_object_set(G_OBJECT(button), "user-data", attachment, NULL);
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(cb_remove_attachment), data);

	snprintf(buf, sizeof(buf), "%.2f %s", attachment->size / (double)get_size_display_factor(attachment->size, &unit), unit);
	size = gtk_label_new(buf);

	
	gtk_box_pack_start (GTK_BOX (hbox), name, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), size, FALSE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
	
	gtk_box_pack_start (GTK_BOX (data->widgets.attachment_box), hbox, FALSE, TRUE, 0);

	
	
}

static void
cb_notebook_prev(GtkButton *button, GtkNotebook *notebook)
{
	gtk_notebook_prev_page(notebook);
}

static void
cb_notebook_next(GtkButton *button, GtkNotebook *notebook)
{
	gtk_notebook_next_page(notebook);
}



static void
cb_attachment_toggle(GtkToggleToolButton *button, GtkNotebook *notebook)
{
	if (gtk_toggle_tool_button_get_active(button))
		gtk_notebook_next_page(notebook);
	else
		gtk_notebook_prev_page(notebook);
}

static void
redraw_attachments(MessagesWritePageData *data)
{
	container_remove_all_widgets(GTK_CONTAINER(data->widgets.attachment_box));
	g_list_foreach(data->message.mail.attachments, 
		       (GFunc) display_attachment,
		       data);
	gtk_widget_show_all(data->widgets.attachment_box);
}

static void
cb_attach (GtkButton *button, MessagesWritePageData *data)
{
	
}

GtkWidget*
messages_create_write_email_page (AppData* app_data, gint action, display_info *message)
{
	
	GtkWidget *page, *toolbar, *scroll, *table, *text;
	GtkWidget *notebook, *vbox, *hbox, *button, *viewport;
	GList *row_list;
	
	MessagesWritePageData *data = g_malloc(sizeof(MessagesWritePageData));
	bzero(data, sizeof(MessagesWritePageData));

	/* Initialising data */

	data->message.mail.message = gtk_text_buffer_new(NULL);
	switch (action) {
		case MESSAGE_WRITE_REPLY_ALL:
			reply(&(data->message), message, TRUE);
			break;
		case MESSAGE_WRITE_REPLY_SENDER:
			reply(&(data->message), message, FALSE);
			break;
		case MESSAGE_WRITE_FORWARD_INLINE:
			forward_inline(&(data->message), message);
			break;
		case MESSAGE_WRITE_FORWARD_ATTACHMENT:
			break;
	}

	/* Initialising widgets */
	page = gtk_vbox_new (FALSE, 0);

	toolbar = gtk_toolbar_new ();
	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (GTK_STOCK_CLOSE, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_message_close), data);
	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (MOKO_STOCK_MAIL_SEND, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_send_email), data);
	
	gtk_box_pack_start (GTK_BOX (page), toolbar, FALSE, TRUE, 0);

	/* We create a vertical notebook with the different input fields */
	/* We place the notebook in a hbox with two buttons right to it, for
	   switching */

	hbox = gtk_hbox_new(FALSE, 0);
	notebook = gtk_notebook_new();
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
	gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), FALSE);

	row_list = NULL;
	row_list = header_table_append_entry(row_list, _("To:"), &(data->widgets.to), data->message.mail.to);
	row_list = header_table_append_entry(row_list, _("Subject:"), &(data->widgets.subject), data->message.mail.subject);
	table = header_table_create(row_list, 0.5f, 1);
	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), table, NULL);
	
	row_list = NULL;
	row_list = header_table_append_entry(row_list, _("Cc:"), &(data->widgets.cc), data->message.mail.cc);
	row_list = header_table_append_entry(row_list, _("Bcc:"), &(data->widgets.bcc), data->message.mail.bcc);
	table = header_table_create(row_list, 0.5f, 3);
	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), table, NULL);
	gtk_box_pack_start (GTK_BOX (hbox), notebook, TRUE, TRUE, 0);

	
	/* Notebook switching */
	vbox = gtk_vbox_new(FALSE, 0);
	button = gtk_button_new();
	gtk_button_set_image(GTK_BUTTON(button), 
			     gtk_image_new_from_icon_name (GTK_STOCK_GO_UP,
							   GTK_ICON_SIZE_MENU));
	gtk_button_set_focus_on_click (GTK_BUTTON(button), FALSE);
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(cb_notebook_prev), notebook);
	gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);
		
	button = gtk_button_new();
	gtk_button_set_image(GTK_BUTTON(button), 
			     gtk_image_new_from_icon_name (GTK_STOCK_GO_DOWN,
							   GTK_ICON_SIZE_MENU));
	gtk_button_set_focus_on_click (GTK_BUTTON(button), FALSE);
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(cb_notebook_next), notebook);
	gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);
	
	gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (page), hbox, FALSE, TRUE, 0);
	


	/* We also create a notebook with attachment and message text and 
	   insert a toolbar toggle for it */


	notebook = gtk_notebook_new();
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
	gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), FALSE);

	toolbar_insert_toggle_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (GREP_FOR_THIS_ATTACHMENT, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_attachment_toggle), notebook);

	/* 1. The message */
	text = gtk_text_view_new_with_buffer(data->message.mail.message);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD_CHAR);
	gtk_container_set_border_width(GTK_CONTAINER(text), 1);
	

	scroll = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll), GTK_SHADOW_IN);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	gtk_container_add (GTK_CONTAINER (scroll), text);
	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), scroll, NULL);
	
	/* 2. The attachments */
	
	vbox = gtk_vbox_new(FALSE, 0);
	/* We create an attachment box, where they are saved. */
	data->widgets.attachment_box = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), data->widgets.attachment_box, TRUE, TRUE, 0);
	redraw_attachments(data);
	
	hbox = gtk_hbox_new(FALSE, 0);
	button = gtk_button_new_with_label(_("Attach..."));
	
	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, TRUE, 0);

	viewport = gtk_viewport_new(NULL, NULL);
	gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);

	scroll = moko_finger_scroll_new();
	gtk_container_add (GTK_CONTAINER (viewport), vbox);	
	gtk_container_add (GTK_CONTAINER (scroll), viewport);
	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), scroll, NULL);

	gtk_box_pack_end (GTK_BOX (page), notebook, TRUE, TRUE, 0);

	data->app_data = app_data;
	data->page = page;
	
	return page;
}
